

#include "processor.inc"
#include "gps_lengths.inc"


;****************************************************************************************
;																						*
;																						*
;			common variable declarations for use by all modules.						*
;          																				*
;			variables are declared here, which reserves room in the RAM					*
;																						*
;			variables are also declared to be global, so the linker can find them		*
;																						*
;			variables are declared as external in the autopilot include file,			*
;			so the linker knows to look													*
;																						*
;																						*
;****************************************************************************************



;********************************************************************************************
;																							*
;	Variables are declared here, in sections.												*
;																							*
;	The first section is "variables", where all subroutine variables should be declared.	*
;	For the bank selection logic to work, there can be no more than 256 "variable" bytes.	*
;																							*
;	The second section, GPSBUFF, is reserved for a buffer used by GPS interrupt handler.	*
;																							*
;	The third section, STACKBUFF, is reserved for a "stack" buffer.							*
;																							*
;********************************************************************************************

VRBLS0		UDATA_ACS	

;	variables used of the LCD display:
ptr_pos		RES 1
ptr_count	RES 1

;	variables used by the delay timers:
temp_1		RES 1
temp_2		RES 1
temp_3		RES 1

;	variables used in the conversion from 8bit byte to decimal ascii display:
LSD		RES 1
MsD		RES 1
MSD		RES 1

	GLOBAL	ptr_pos, ptr_count, temp_1, temp_2, temp_3, LSD, MsD, MSD

;	variables used in the conversion from 16bit, 2 byte to decimal ascii display
NumH		RES 1
NumL		RES 1
Sign		RES 1
TenK		RES 1
Thou		RES 1
Hund		RES 1
Tens		RES 1
Ones		RES 1

	GLOBAL	NumH, NumL, Sign, TenK, Thou, Hund, Tens, Ones

;	variables used in the pulse width modulation routines

;
;	timer 2 generates a fast PWM "heartbeat", approximately one every millisecond.
;	most (PWM_skip) timer 2 interrupts are skipped, producing no pulses.
;	PWM_count is used to count timer 2 interrupts

PWM_count	RES	1	; a count of timer2 interrupts
PWM_skip	RES	1	; how many timer2 interrupts should be ignored between servo pulses.
PWM_goto_L	RES	1	; used in computed goto in PWM

PWM1_dc		RES	1	; the PWM1 duty cycle, computed by the steering algorithm.
PWM2_dc		RES	1	; 0 produces a net servo pulse of around 1 msec, for hard left (or full off).
					; 255 produces a net servo pulse of around 2 msect, for hard right (or full on).

PORTB_snap	RES	1	; snapshot of B port
PORTB_old	RES	1	; previous value
PORTB_xor	RES	1	; changes bits

;	timer 2 is also used to decide when to read the compass, once every 100 milliseconds.
;	CMP_skip is the number of servo pulses to skip between compass readings.
;	CMP_count is used to count servo pulses


	GLOBAL PWM_count, PWM_skip, PWM_goto_L, PWM1_dc, PWM2_dc
	GLOBAL PORTB_snap, PORTB_old, PORTB_xor

;	variables used to service the GPS receiver

GPS_char		RES	1	; used to accept a character from the serial receiver
state_flags		RES	1	; collection of control bits to interface across interrupt
GPS_ps_char		RES 1	; used to handle one character during display of GPS info
GPS_bf_pnt		RES	1	; used to preserve the GPS buffer pointer between interrupts
GPS_item_size	RES 1	; size of a GPS buffer item
VTG_item_size	RES	1	; size of a VTG item



;	the following are comma terminated ascii strings parsed from the GPS chip output

GPS_status		RES GPS_statusR	; the GPS status reported by the GPS chip in rms message
GPS_fix			RES GPS_fixR	; the GPS fix reported by the GSP chip in gga message
GPS_long		RES GPS_longR	; longitude
GPS_long_bin	RES	2			; binary longitude
GPS_EW			RES	GPS_EWR		; east west indicator
GPS_lat			RES GPS_latR 	; latitude
GPS_lat_bin		RES 2			; binary latitude
GPS_NS			RES	GPS_NSR		; north south indicator
GPS_sat			RES	GPS_satR	; the number of satellites in view
GPS_alt			RES	GPS_altR	; altitude
GPS_course		RES GPS_courseR
GPS_speed		RES GPS_speedR
orig_long_bin	RES	2			; origin longitude binary
orig_lat_bin	RES 2			; origin latitude binary
digit_5_roll	RES	1			; digit 5 rollover difference


	GLOBAL	GPS_char, GPS_ps_char, GPS_status, GPS_fix
	GLOBAL	GPS_long, GPS_lat
	GLOBAL	GPS_course , GPS_speed
	GLOBAL	GPS_sat, GPS_alt, GPS_EW, GPS_NS, GPS_buffer, GPS_bf_pnt, GPS_item_size, VTG_item_size
	GLOBAL	orig_long, orig_lat
	GLOBAL	GPS_long_bin, GPS_lat_bin, orig_long_bin, orig_lat_bin, digit_5_roll
	GLOBAL	state_flags


errorDir		RES	1	; desiredDir minus actualDir

strngTrim		RES	1	; steering trim

elevTrim		RES	1	; elevator trim

;	variables used in measuring and counting input rudder, elevator, and select pulses

tmrudinH		RES	1	;	rudder input time
tmrudinL		RES	1	;	

tmeleinH			RES	1	;	elevator input time
tmeleinL			RES	1	;	

tmselinH		RES	1	;	select input time
tmselinL		RES	1	;	


tmr1Lsnap		RES	1	;	snapshot of timer 1
tmr1Hsnap		RES	1	;	

tmrnible		RES	1	;	timer nibble temporary variable


	GLOBAL	tmrudinH, tmrudinL, tmeleinH, tmeleinL, tmselinH ,tmselinL
	GLOBAL	pwrudin, pwelein, pwselin, pwrudfilH, pwrudfilL
	GLOBAL	pulsesrudin, pulseselein, pulsesselin
	GLOBAL	tmr1Lsnap, tmr1Hsnap, tmrnible

; odds and ends used some of the utilities provided with the demo

COUNTER		res	1
delay		res	1
temp_wr		res	1
temp_rd		res	1

	GLOBAL	COUNTER, delay, temp_wr, temp_rd


avail_flags		RES 1	; flags used to determine whether or not to arm the autopilot
cntrl_flags		RES	1	; overall state control flags

	GLOBAL	avail_flags, cntrl_flags

X_tempH		RES	1
X_tempL		RES	1
Y_tempH		RES	1
Y_tempL		RES	1

	GLOBAL	X_tempH, X_tempL, Y_tempH, Y_tempL

theta_temp	RES	1
delta_theta	RES	1

	GLOBAL	theta_temp, delta_theta


VRBLS1		UDATA

pwrudin			RES	1	;	pulse width rudder input

pwrudfilH		RES	1	;	filtered pw rudder input
pwrudfilL		RES	1

pwelein			RES	1	;	pulse width elevator input
pwselin			RES	1	;	pulse width select input

pulsesrudin		RES	1	;	pulse count rudder input
pulseselein		RES	1	;	pulse count elevator input
pulsesselin		RES	1	;	pulse count select input


;	variables used in trig table lookup

THETA	RES	1
COSINEH	RES	1
COSINEL	RES	1
SINEH	RES	1
SINEL	RES	1

	GLOBAL	THETA, COSINEH, COSINEL, SINEH, SINEL

;	variables used in math multiply software library

ARG1H		RES 1
ARG1L		RES 1
ARG2H		RES 1
ARG2L		RES	1
RES3		RES 1
RES2		RES 1
RES1		RES 1
RES0		RES 1

	GLOBAL	ARG1H, ARG1L, ARG2H, ARG2L, RES3, RES2, RES1, RES0

;	variables used in rotation of an xy pair

X_rectH		RES	1
X_rectL		RES	1
Y_rectH		RES	1
Y_rectL		RES	1

	GLOBAL	X_rectH, X_rectL, Y_rectH, Y_rectL


;	variables used by steering.
; 	direction is represented clockwise by a single byte,
;	zero is north, 64 is east, 128 is south, 192 is west, etc.

desiredDir	RES	1	; desired direction of travel
actualDir	RES 1	; corrected compass reading


strngGain	RES	1	; signed steering gain, with a 4 bit binary point,
					; that is, 00010000 represents a gain of 1 in engineering units


;desiredSpeed	RES	1	; desired speed

	
	GLOBAL	desiredDir, actualDir, errorDir, strngGain, strngTrim, elevTrim
;	GLOBAL  desiredSpeed

origDir			RES	1			; direction the compass was pointed when the origin was set

xWorldH			RES 1			; x position from origin, world co-ordinates
xWorldL			RES	1
yWorldH			RES	1			; y position from origin, world co-ordinates
yWorldL			RES	1

xFocusH			RES	1			; x position from origin, second focus
xFocusL			RES	1
yFocusH			RES	1			; y position from origin, second focus
yFocusL			RES	1

course			RES	1			; GPS course
speedH			RES	1			; GPS speed, H
speedL			RES	1			; GPS speed, L

	GLOBAL	xWorldH, xWorldL, yWorldH, yWorldL, origDir
	GLOBAL	xFocusH, xFocusL, yFocusH, yFocusL
	GLOBAL	course, speedH, speedL

ellipseConstH	RES	1
ellipseConstL	RES	1

angle1			RES	1
angle2			RES	1
angleTang		RES	1
dist1H			RES	1
dist1L			RES	1
dist2H			RES	1
dist2L			RES	1

	GLOBAL	ellipseConstH , ellipseConstL, angle1, angle2, angleTang, dist1H, dist1L, dist2H, dist2L

xaccelU			RES	1	;	filtered acceleration along yaw axis
xaccelH			RES	1
xaccelL			RES	1

yaccelU			RES	1	;	filtered acceleration along roll axis
yaccelH			RES	1
yaccelL			RES	1

refU			RES	1	;	filtered reference voltage for gyros
refH			RES	1
refL			RES	1

pitchU			RES	1	;	filtered pitch gyro
pitchH			RES	1
pitchL			RES	1

elevU			RES	1	;	elevator control accumulator
elevH			RES	1
elevL			RES	1

yawunfH			RES	1	;	unfiltered yaw gyro
yawunfL			RES	1

yawsqU			RES	1	;	filtered yaw square
yawsqH			RES	1
yawsqL			RES	1

yawU			RES	1	;	filtered yaw gyro
yawH			RES	1
yawL			RES	1

ruddU			RES	1	;	rudder control accumulator
ruddH			RES	1
ruddL			RES	1

strngTrimTemp		RES	1	; steering trim, final step

speedTrimTemp		RES	1	; speed trim, final step



	GLOBAL	xaccelU, xaccelH, xaccelL
	GLOBAL	yaccelU, yaccelH, yaccelL
	GLOBAL	refU, refH, refL
	GLOBAL	pitchU, pitchH, pitchL
	GLOBAL	elevU, elevH, elevL
	GLOBAL	yawU, yawH, yawL
	GLOBAL	ruddU, ruddH, ruddL
	GLOBAL	strngTrimTemp, speedTrimTemp
	GLOBAL	yawunfH , yawunfL
	GLOBAL	yawsqU , yawsqH, yawsqL

yawadjH			RES	1	;	raw, adjusted, yaw gyro
yawadjL			RES	1

yawfilU			RES	1	;	yaw filtered for bank angle
yawfilH			RES	1
yawfilL			RES	1

ruddunfH		RES	1	;	unfiltered rudder, for mixing
ruddunfL		RES	1

ruddfilU		RES	1	;	filtered rudder
ruddfilH		RES	1
ruddfilL		RES	1

infilH			RES	1	; input to filter
infilL			RES	1

outfilU			RES	1	; filter state variable
outfilH			RES	1
outfilL			RES	1

taufil			RES	1	; 256*dt/tau

	GLOBAL		yawadjH , yawadjL , yawfilU, yawfilH , yawfilL
	GLOBAL		ruddunfH, ruddunfL, ruddfilU , ruddfilH , ruddfilL
	GLOBAL		infilH , infilL , outfilU , outfilH , outfilL 
	GLOBAL		taufil

;	self calibrating baselines, which are snapshots of filtered values on startup:

pitch0U			RES	1
pitch0H			RES	1

yaw0U			RES	1
yaw0H			RES	1

xaccel0U		RES	1
xaccel0H		RES	1

yaccel0U		RES	1
yaccel0H		RES	1

yaccelmavL		RES	1
yaccelmavH		RES	1

ref0U			RES	1
ref0H			RES	1

requests		RES	1

	GLOBAL		pitch0U, pitch0H, yaw0U, yaw0H
	GLOBAL		xaccel0U, xaccel0H, yaccel0U, yaccel0H, yaccelmavL, yaccelmavH
	GLOBAL		ref0U, ref0H, requests

mixgain			RES	1
mixgyr			RES	1
pitchgain		RES	1
accelgain		RES	1
yawgain			RES	1
auggain			RES	1

	GLOBAL		mixgain , mixgyr, pitchgain, accelgain, yawgain, auggain

;fw_count		RES	1
	
;	GLOBAL		fw_count

calib_timer		RES	1	;	calibration timer
standby_timer	RES	1	;	standby timer
waggle			RES	1	;	rudder waggle during GPS acquire

	GLOBAL		calib_timer, standby_timer , waggle


;	the buffer used to store GPS strings as they come in.
;	note that the buffer is in the section of RAM called "GPSBUFF".

GPSBUFF		UDATA
GPS_buffer	RES .100
orig_long		RES	orig_longR	; origin longitude
orig_lat		RES	orig_latR	; origin latitude


	GLOBAL	GPS_buffer

;	the buffer used as a data stack by the interrupt service routines.
;	note that the buffer is in the section of RAM called "STACKBUFF".

STACKBUFF	UDATA
stack_buff	RES	.256

	GLOBAL	stack_buff

	end
