#include "gps_lengths.inc"

#include "options.inc"


;****************************************************************************
;																			*
;	defines, externs, and macro definitions that are included in all files.	*
;																			*
;****************************************************************************

;	definition of push buttons and LEDS:

#define	LatGain		.37
#define	LongGain	.27

#define LatMod		(LatGain*.1000)
#define LongMod		(LongGain*.1000)

#define LatModH		(LatMod/.256)
#define LatModL		(LatMod-.256*LatModH)

#define LongModH	(LongMod/.256)
#define LongModL	(LongMod-.256*LongModH)


#define	pwmin2_dir	TRISB,4
#define	pwmin2		PORTB_snap,4
#define pwmin2_xor	PORTB_xor,4

#define	pwmin1_dir	TRISB,5
#define	pwmin1		PORTB_snap,5
#define pwmin1_xor	PORTB_xor,5

#define	pwmout2_dir	TRISC,1
#define pwmout2		PORTC,1

#define	pwmout1_dir	TRISC,2
#define pwmout1		PORTC,2

#ifdef	version1
#include	"version1.inc"
#endif

#ifdef	version2
#include	"version2.inc"
#endif

#ifdef	version3
#include	"version3.inc"
#endif

#ifdef	sf252v1
#include	"sf252v1.inc"
#endif

#ifdef	sf2520v1
#include	"sf2520v1.inc"
#endif

#ifdef	sf2520v1a
#include	"sf2520v1a.inc"
#endif

;	definition of GPS control bit locations on the GPS_state byte:

#define	GPS_dollar		state_flags,0	;	$ received from GPS
#define VTG_end			state_flags,1	;	2nd message, VTG, received from GPS
#define GPS_rq_org		state_flags,2	;	request to capture origin
#define	GPS_org_rdy		state_flags,3	;	origin was captured
#define GPS_verbose		state_flags,4	;	GPS has sent extra messages
#define GPS_req			state_flags,5	;	request to parse GPS message
#define	GPS_tracking	state_flags,6	;	GPS is tracking
#define GPS_valid		state_flags,7	;	GPS message was understood

#define nav_capable		avail_flags,0	;	navigation has everything it needs
#define	nav_busy		avail_flags,1	;	navigation is busy, block re-entrant call
#define	GPS_steering	avail_flags,2	;	use GPS to steer
#define	circle_req		avail_flags,3	;	request to circle
#define auto_req		avail_flags,4	;	request to try to run in automatic mode
#define radio_on		avail_flags,5	;	RC transmitter is on
#define man_req			avail_flags,6	;	request for manual mode
#define enable_focus	avail_flags,7	;

#define	startM			cntrl_flags,0	;
#define	calibrateM		cntrl_flags,1	;	
#define	acquiringM		cntrl_flags,2	;	
#define manualM			cntrl_flags,3
#define autoM			cntrl_flags,4
#define returnM			cntrl_flags,5
#define circlingM		cntrl_flags,6
#define cntrlF7			cntrl_flags,7	;

#define	calibpitch		requests,0	;	request to calibrate pitch gyro
#define	calibref		requests,1	;	request to calibrate the CPU A/D
#define	calibyaw		requests,2	;	request to calibrate the yaw gyro
#define	calibxaccel		requests,3	;	request to calibrate the x accelerometer
#define	calibyaccel		requests,4	;	request to calibrate the y accelerometer
#define	RUD_req			requests,5	;	request to compute rudder
#define	ELE_req			requests,6	;	request to compute elevator
#define	test_mode		requests,7	;	test_mode


;	definition of offsets into longitude, latidude

#define long_d0	.9
#define long_d1 .8
#define long_d2 .7
#define long_d3 .6
#define long_d4 .4
#define long_d5 .3
#define long_d6 .2
#define long_d7 .1
#define long_d8 .0

#define lat_d0 .8
#define lat_d1 .7
#define lat_d2 .6
#define lat_d3 .5
#define lat_d4 .3
#define lat_d5 .2
#define lat_d6 .1
#define lat_d7 .0

;	definition of baud rate register values for 4.8kbaud:
;	(we really should have something like this for I2C for compass, too.)

#define ubrg_4p8_4	.51			; usart brg, 4.8k at 4 Mz
#define ubrg_4p8_16	.207		; usart brg, 4.8k at 16 Mz

#define	i2cbrg_100_16 .40		; i2c brg, 100k at 16 Mz

#define minSpeed	.20	; 20 times minimum speed to compute compass correction from GPS doppler shift

;	negative is forward

#define	slowSpeed -.10

#define mediumSpeed -.20

#define circleGain .100

#define	radius15m .30

#define radius10m .20

;	ALL variables must be declared external here so that the linker will know to look for them
;	outside of the modules where they are used. 

	EXTERN	ptr_pos, ptr_count, temp_1, temp_2, temp_3, LSD, MsD, MSD
	EXTERN	NumH, NumL, Sign, TenK, Thou, Hund, Tens, Ones

	EXTERN	ARG1H, ARG1L, ARG2H, ARG2L, RES3, RES2, RES1, RES0
	EXTERN	THETA, COSINEH, COSINEL, SINEH, SINEL

	EXTERN	X_rectH, X_rectL, Y_rectH, Y_rectL
	EXTERN	X_tempH, X_tempL, Y_tempH, Y_tempL
	EXTERN	theta_temp, delta_theta

	EXTERN 	PWM_count, PWM_skip, PWM_goto_L, PWM1_dc, PWM2_dc
	EXTERN	PORTB_snap, PORTB_old, PORTB_xor

	EXTERN	stack_buff

	EXTERN	desiredDir, actualDir, errorDir, strngGain

	EXTERN	state_flags, avail_flags, cntrl_flags

	EXTERN	GPS_char, GPS_ps_char, GPS_status, GPS_fix
	EXTERN	GPS_long, GPS_lat
	EXTERN	GPS_course , GPS_speed
	EXTERN	GPS_sat, GPS_alt, GPS_EW, GPS_NS, GPS_buffer, GPS_bf_pnt, GPS_item_size, VTG_item_size
	EXTERN 	GPS_long_bin, GPS_lat_bin

	EXTERN	orig_long, orig_lat
	EXTERN	origDir, orig_long_bin, orig_lat_bin
	EXTERN	digit_5_roll
	EXTERN	xWorldH, xWorldL, yWorldH, yWorldL
	EXTERN	xFocusH, xFocusL, yFocusH, yFocusL

	EXTERN	course, speedH, speedL

	EXTERN	ellipseConstH , ellipseConstL, angle1, angle2, angleTang, dist1H, dist1L, dist2H, dist2L


	EXTERN	COUNTER
	EXTERN	delay, temp_wr, temp_rd

	EXTERN	strngTrim, elevTrim
;	EXTERN	desiredSpeed


	EXTERN	xaccelU, xaccelH, xaccelL
	EXTERN	yaccelU, yaccelH, yaccelL
	EXTERN	refU, refH, refL
	EXTERN	pitchU, pitchH, pitchL
	EXTERN	elevU, elevH, elevL
	EXTERN	yawU, yawH, yawL
	EXTERN	ruddU, ruddH, ruddL
	EXTERN	strngTrimTemp, speedTrimTemp

	EXTERN	yawunfH , yawunfL
	EXTERN	yawsqU , yawsqH , yawsqL

	EXTERN	yawadjH , yawadjL , yawfilU, yawfilH , yawfilL
	EXTERN	ruddunfH , ruddunfL , ruddfilU, ruddfilH , ruddfilL
	EXTERN	infilH , infilL , outfilU , outfilH , outfilL 
	EXTERN	taufil

	EXTERN	tmrudinH, tmrudinL, tmeleinH, tmeleinL, tmselinH ,tmselinL
	EXTERN	pwrudin, pwelein, pwselin, pwrudfilH , pwrudfilL
	EXTERN	pulsesrudin, pulseselein, pulsesselin
	EXTERN	tmr1Lsnap, tmr1Hsnap, tmrnible

	EXTERN	pitch0U, pitch0H, yaw0U, yaw0H
	EXTERN	xaccel0U, xaccel0H, yaccel0U, yaccel0H, yaccelmavL, yaccelmavH
	EXTERN	ref0U, ref0H, requests

	EXTERN	mixgain , mixgyr, pitchgain, accelgain, yawgain, auggain

;	EXTERN	fw_count

	EXTERN	calib_timer, standby_timer , waggle


;	macro that came with the original microchip demo
;	for determining when the SSP (I2C) module is idle.
;	5 lines have been commented out, because they don't seem to do anything.
;	As it stands, the macro simply waits for transmitted data to be done:

ssprw	macro				;check for idle SSP module routine
;	movlw	0x00
;	andwf	SSPCON2,W
;	sublw	0x00
;	btfss	STATUS,Z
;	bra	$-8
	btfsc	SSPSTAT,R_W
	bra	$-2
	endm

;	macro for loading a table pointer:

ld_tblptr	macro	TABLE_ADDRESS
	movlw	HIGH TABLE_ADDRESS
	movwf	TBLPTRH
	movlw	LOW TABLE_ADDRESS
	movwf	TBLPTRL
	endm

;	macro for generating inline code for an 8X8 signed multiply:

smult	macro	arg1 , arg2
	movf	arg1, W
	mulwf	arg2
	btfsc	arg2, 7
	subwf	PRODH, F
	movf	arg2, W
	btfsc	arg1, 7
	subwf	PRODH, F
	endm

scale	macro factor , parameter , destination
	movf	factor , W
	mullw	parameter
#if	( ! (( parameter & 0x80 ) == 0 ))
	subwf	PRODH , F
#endif
	movff	PRODH , destination
	endm

echo_bit	macro	input_r, input_b , output_r, output_b
	btfsc	input_r, input_b
	bsf		output_r , output_b
	btfss	input_r , input_b
	bcf		output_r , output_b
	endm

flip_bit	macro	input_r, input_b , output_r, output_b
	btfsc	input_r, input_b
	bcf		output_r , output_b
	btfss	input_r , input_b
	bsf		output_r , output_b
	endm

doubleres	macro
	movf	RES0 , w
	addwf	RES0 , f
	movf	RES1 , w
	addwfc	RES1 , f
	movf	RES2 , w
	addwfc	RES2 , f
	endm

movlf		macro	source , dest
	movlw	source
	movwf	dest
	endm
