

#include "processor.inc"
#include "autop_defines.inc"



;************************************************************************************
;																					*
;	This is where the one second processing goes.									*
;																					*
;	one_sec_tasks gets called precisely once per second,							*
;	triggered by the GPS clock.														*
;																					*
;	Prior to entry, the GPS data has been read and parsed.							*
;																					*
;	This is where the call will be made to Andrew and Brian's navigation routine.	*
;																					*
;************************************************************************************


;************* Note: the entry point is part way down, "one_sec_tasks" ***************


	EXTERN	navigate, rect_to_polar, MULS2X2 , save_focus

PROG1 	CODE

#define	xWorld	xWorldH
#define yWorld  yWorldH

;
;	The following code is used to make a necessary "modulo" adjustment when
;	the origin is close to a "rollover" value, with a bunch of 0's.


add_long_corr
	movlw	LongModL
	addwf	xWorld+1 , F
	movlw	LongModH
	addwfc	xWorld , F
	bra	skip_long_corr

sub_long_corr
	movlw	LongModL
	subwf	xWorld+1 , F
	movlw	LongModH
	subwfb	xWorld , F
	bra	skip_long_corr

add_lat_corr
	movlw	LatModL
	addwf	yWorld+1 , F
	movlw	LatModH
	addwfc	yWorld , F
	bra	skip_lat_corr

sub_lat_corr
	movlw	LatModL
	subwf	yWorld+1 , F
	movlw	LatModH
	subwfb	yWorld , F
	bra skip_lat_corr

one_sec_tasks


;
;	Start by subtracting the origin from the GPS coordinate.
;
;	So, xyWorld = GPS_long_lat_bin - origin_long_lat_bin.
;


	movff	GPS_long_bin , xWorld
	movff	GPS_long_bin+1 , xWorld+1
	movff	GPS_lat_bin , yWorld
	movff	GPS_lat_bin+1 , yWorld+1

	movf	orig_long_bin+1 , W
	subwf	xWorld+1 , F
	movf	orig_long_bin , W
	subwfb	xWorld , F

	movf	orig_lat_bin+1 , W
	subwf	yWorld+1 , F
	movf	orig_lat_bin , W
	subwfb	yWorld , F

;
;	We look to the 5th digit of the GPS ASCII value to see if there is a rollover.
;	If so, we must apply corrections. Most combinations we simply look at the digit difference.
;	But, be careful with the 0, 9 digit combination!! (Brian fixed a mistake I made.)
;
	movff	orig_long+long_d5 , WREG
	subwf	GPS_long+long_d5 , W
	bz		skip_long_corr
	movwf	digit_5_roll
	btfsc	WREG, 7
	negf	WREG
	sublw	.9
	bz		zero_nine_long
	movf	digit_5_roll , W
	bn		add_long_corr
	bra		sub_long_corr

zero_nine_long
	movf	digit_5_roll , W
	bn		sub_long_corr
	bra		add_long_corr
	

skip_long_corr

	movff	orig_lat+lat_d5 , WREG
	subwf	GPS_lat+lat_d5 , W
	bz		skip_lat_corr
	movwf	digit_5_roll
	btfsc	WREG, 7
	negf	WREG
	sublw	.9
	bz		zero_nine_lat
	movf	digit_5_roll , W	
	bn		sub_lat_corr
	bra		add_lat_corr

zero_nine_lat
	movf	digit_5_roll , W
	bn		add_lat_corr
	bra		sub_lat_corr

skip_lat_corr

;	navigation

	btfsc	enable_focus
	call	save_focus

	btfss	GPS_steering	;	navigate only when needed
	return


	bsf		nav_busy	;	block re-entrant call through 1 second task scheduler

	
	
	call	navigate	;	figure out where you are and what direction to go!


	bcf		nav_busy	;	re-enable call

	return


	GLOBAL	one_sec_tasks

	end	
