
#include "processor.inc"
#include "autop_defines.inc"

;************************************************************************************
;																					*
;	The module for reading the compass.												*
;																					*
;	The reader is referred to the documentation on I2C, as well as the 				*
;	compass documentation. The interface to the compass is I2C.						*
;																					*
;	I2C is "inter-integrated circuit" communications protocol, developed by 		*
;	Philips. It is a 2 wire, bi-direction, synchronous bus communication protocol.	*
;																					*
;	To communicate with the compass, you go through the following steps:			*
;																					*
;	1. Stop the bus, in case the compass is confused.								*
;	2. Start the bus, to get the attention of the slaves on the bus.				*
;	3. Send out the write address of the compass to let it know something is coming.*
;	4. Send out the command "1", to let the compass know you want its first register*
;	5. Send out a repeated send on the bus, to let everyone know more is coming.	*
;	6. Send out the read address of the compass to let it know you want something.	*
;	7. Receive the direction information from the compass.							*
;																					*
;	Each exchange of data is done with "acknowledge" or "not acknowledge"			*
;	to handshake the exchange.														*
;																					*
;************************************************************************************

	EXTERN LCDLine_1 , d_write, bin_bcd

PROG1 	CODE

CMPSInit

;	turn on the clock and data pins:

;	bsf	TRISC,3			;initialize MSSP module
;	bsf	TRISC,4

;	setup the control registers:

;	movlw	B'00101000'	;setup the SSP module as I2C communications
	movlw	B'00000000'
	movwf	SSPCON1
;	bsf		SSPSTAT,SMP			;slew control disabled
;	movlw	i2cbrg_100_16	;
;	movwf	SSPADD

	clrf	vtg_adj

	return



get_bearing

#if ( compass == 0 )

	bsf		compass_on
	return

#else

			
	bsf	SSPCON2,PEN		;stop the bus
	btfsc	SSPCON2,PEN
	bra	$-2

	bsf	SSPCON2,SEN		;start the bus
	btfsc	SSPCON2,SEN
	bra	$-2 	
	movlw	B'11000000'		;send CMPS03 write ADDRESS
	movwf	SSPBUF
	ssprw

	btfsc	SSPCON2,ACKSTAT		;is everything OK? 
	return

	movlw	0x01				; 01 signifies we want the 1 byte bearing data
	movwf	SSPBUF				; send the command to the compass
	ssprw						; wait till its gone
	btfsc	SSPCON2,ACKSTAT		;is everything OK?
	return

	bsf	SSPCON2,RSEN		;send repeated start
	btfsc	SSPCON2,RSEN
	bra	$-2
	movlw	B'11000001'		;send CMPS03 read ADDRESS
	movwf	SSPBUF
	ssprw				;module idle?
	btfsc	SSPCON2,ACKSTAT		;is everything OK?
	return

	bsf		SSPCON2,RCEN		;enable receive mode
	btfsc	SSPCON2,RCEN
	bra	$-2

	movf	SSPBUF , W				; retrieve compass direction
	negf	WREG					; flip sign

;
;	compass corrections and offsets:
;	north is 90 degrees
;	compass is mounted 90 degrees
;	in this area, compass correction is about 14 degrees

	addlw	0x40-compass_offset		; north is 90 degrees
	movwf	compassDir

#if	( cmpsLED == 1)
	swapf	WREG
	movwf	PORTA
	swapf	WREG
#endif
	
	bsf		SSPCON2,ACKDT		;send NOT-ACK to complete the transaction
	bsf		SSPCON2,ACKEN
	btfsc	SSPCON2,ACKEN
	bra	$-2
			
	bsf		SSPCON2,PEN		;stop the I2C bus
	btfsc	SSPCON2,PEN
	bra	$-2

	addwf	vtg_adj , W
	movwf	actualDir

	bsf	compass_on

	return

#endif

;;************** Display compass heading info on the LCD *************
;
;display_hdg					;	entry point from the menu driver
;	btfss	scroll			;	wait for RA4 release
;	bra		$-2		
;	btfss	select			;	wait for RB0 release
;	bra		$-2

;disp_hdg_again				;	display loop

;	btfss	select
;	movff	actualDir, desiredDir

;	rcall	LCD_dsp_hdg		;	display the long, lat, direction and satellite data
;	btfsc	scroll			;	presss RA4 to get out
;	bra		disp_hdg_again
;	btfss	scroll			;	release RA4 to complete
;	bra		$-2
;	goto	menu_gps

;LCD_dsp_hdg
			
;	call	LCDLine_1				;	point LCD cursor to beginning of line 1


;	movf	actualDir , W			;	get the compass reading
;	call	bin_bcd					;	convert to ASCII

;	movf	MSD , W					;	display first character of direction
;	movwf	temp_wr
;	call	d_write

;	movf	MsD , W					;	display second character of direction
;	movwf	temp_wr
;	call	d_write

;	movf	LSD , W					;	display third charater of direction
;	movwf	temp_wr
;	call	d_write

;	return



	GLOBAL	get_bearing, CMPSInit


	end	
