;************************************************************************
;																		*
;		Initialization and menu controls.								*
;																		*
;		Basically, this is the main program.							*
;		It sets everything up for the interrupt handlers,				*
;		and then provides background interaction via some menus.		*
;																		*
;		Background interaction steps through a series of sections.		*
;
;*		 																*
;*																		*
;*		1.	Read GPS													*
;*			This function tests reading the GPS.					 	*
;*			Whenver new GPS information is available,					*
;*			latitude, compass, longitude, and number of satellites		*
;*			are displayed.												*
;*																		*
;*																		*
;*																		*
;************************************************************************

#include "processor.inc"
#include "autop_defines.inc"



;	routines referenced in this module:

	EXTERN	LCDInit, d_write, i_write, LCDLine_1, LCDLine_2
	EXTERN	stan_char_1, stan_char_2, delay_1s
	EXTERN	display_GPS, display_XY, set_gxx, display_vtg
	EXTERN	serv_PWM, PWMInit, GPSInit, SteerInit, TMR0Init
	EXTERN	ADInit

PROG1 	CODE

;
;	The following is a data table used to store strings
;	that are sent to the LCD display.
;	Each string must be exactly 16 bytes long.
;	Strings are referred to by their offsets into the table.
;	There are 16 entries in the table.
;	Unused entries are indicated.

stan_table				;table for standard code
	;	"XXXXXXXXXXXXXXXX"
	;				ptr:
	data	"                "	;0 - clears the display
	data	"Tggl tst        "	;16
	data	"Dsply vtg       "	;32
	data	"Set origin      "	;48
	data	"RA4=Next RB0=Now"	;64
	data	"Ver. 04/11/05   "  ;80
	data	"Come home       "	;96
	data	"Set focus xy    "	;112
	data	"Unused128       "	;128
	data	"Unused144       "	;144
	data	"Unused160       "	;160
	data	"Unused176       "	;176
	data	"Unused192       "  ;192
	data	"Unused208       "  ;208
    data    "Unused224       "  ;224
    data    "Unused240       "  ;240
;
;	This is where it all begins. On power up, or reset, this is where the CPU starts.

start
	movlw	0x01				;	setup the bank selection register for bank1 variables.
	movwf	BSR
	movlw	0x0					;	initialize table pointer 3rd most significant byte
	movwf	TBLPTRU

;	Initialize all of the modules

	call	TMR0Init
	call	GPSInit				;	initialize the GPS module
	call	PWMInit				;	initialize the servo PWM module.

	LFSR	FSR2 , 	stack_buff	;	initialize the stack buffer pointer
	LFSR	FSR1 ,	GPS_lat		;	initialize the GPS parsing pointer


#ifdef  version1
	call 	LCDInit				;	initialize the LCD display
#endif

	call	SteerInit			;	initialize the steering module
	call	ADInit				;	initialize the AD

;	Set the Input/Output direction on some of the pins,
;	and initialize them. debugA, B, and C are the debug LEDs.

	bcf		INTCON2, RBPU	; enable the pullup resistors on PORTB

#ifdef	scroll_dir		
	bsf		scroll_dir			;make scroll switch an Input
#endif

#ifdef	select_dir
	bsf		select_dir			;make select switch an Input
#endif

#ifdef	SWR1_dir				;	SR1 switch input
	bsf		SWR1_dir
#endif

#ifdef	SWR2_dir				;	SR2 switch input
	bsf		SWR2_dir
#endif

#if ( cpuUsage==0)
	bcf		debugA_dir	; output
	bcf		debugA		; off
	bcf		debugB_dir	; output
	bcf		debugB		; off
#else
	bsf		debugA_dir
	bsf		debugB_dir
#endif

#ifdef	debugC
	bcf		debugC_dir	; output
	bcf		debugC		; off
#endif

	bsf		pwmin1_dir	; input
	bsf		pwmin2_dir	; input

	bcf		pwmout2_dir	; output
	bcf		pwmout2		; off
	bcf		pwmout1_dir	; output
	bcf		pwmout1		; off

	bsf		pwmin3r_dir	; input
	bsf		pwmin3f_dir	; input

#ifdef	st3st1
	bcf	st3st1_dir	;	output
	bcf	st3st1		;	self test off
#endif

#ifdef	st2
	bcf	st2_dir		;	output
	bcf	st2			;	self test off
#endif

#if ( GPSDEBUG == 1)
	bcf		TRISC,6	;
#endif
		
;	Setup the configuration bits for the interrupts

	bsf		RCON, IPEN		; enable high/low priority structure

	bcf		IPR1, RCIP		; USART low priority
	bsf 	PIE1, RCIE		; USART enable
	bcf		PIR1, RCIF		; clear any stale interrupt request, USART

	bcf		IPR1, TMR2IP	; PWM timer 2, low priority
	bsf 	PIE1, TMR2IE	; PWM timer 2, enable
	bcf		PIR1, TMR2IF	; clear any stale interrupt request, PWM timer 2

	bsf 	INTCON2, TMR0IP	; timer0 watchdog, high priority
	bsf 	INTCON, TMR0IE	; timer0 watchdog, enable
	bcf 	INTCON, TMR0IF	; clear any stale interrupt request, timer 0

#ifdef sf252v1

	bsf		INTCON3, INT1IP	; interrupt 1, high priority, PWM measurement
	bsf		INTCON3, INT1IE	; interrupt 1, enable
	bcf		INTCON3, INT1IF	; clear any stale interrupt request, int 1
	bsf		INTCON2, INTEDG1	; interrupt 1 on rising edge

	bsf		INTCON3, INT2IP		; interrupt 2, high priority, PWM measurement
	bsf		INTCON3, INT2IE		; interrupt 2, enable
	bcf		INTCON3, INT2IF		; clear any stale interrupt request, int 2
	bcf		INTCON2, INTEDG2	; interrupt 2 on falling edge

#endif


#ifdef sf2520v1

	bsf		INTCON3, INT1IP	; interrupt 1, high priority, PWM measurement
	bsf		INTCON3, INT1IE	; interrupt 1, enable
	bcf		INTCON3, INT1IF	; clear any stale interrupt request, int 1
	bsf		INTCON2, INTEDG1	; interrupt 1 on rising edge

	bsf		INTCON3, INT2IP		; interrupt 2, high priority, PWM measurement
	bsf		INTCON3, INT2IE		; interrupt 2, enable
	bcf		INTCON3, INT2IF		; clear any stale interrupt request, int 2
	bcf		INTCON2, INTEDG2	; interrupt 2 on falling edge

#endif


#ifdef sf2520v1a

	bsf		INTCON, INT0IE	; interrupt 1, enable
	bcf		INTCON, INT0IF	; clear any stale interrupt request, int 1
	bsf		INTCON2, INTEDG0	; interrupt 1 on rising edge

	bsf		INTCON3, INT1IP		; interrupt 2, high priority, PWM measurement
	bsf		INTCON3, INT1IE		; interrupt 2, enable
	bcf		INTCON3, INT1IF		; clear any stale interrupt request, int 2
	bcf		INTCON2, INTEDG1	; interrupt 2 on falling edge

#endif


	bsf		INTCON2, RBIP		; port b interrupt on value, high priority
	bcf		INTCON, RBIF		; clear any stale interrupt request, portb

	bcf		PIE1, TMR1IE	; timer1, disable interrupt
	movlw	B'10000000'		; 1 = timer1, 16 bit operation
							; 0 = not used
							; 0 = 1:1 prescaler
							; 0 = not used
							; 0 = T1 osc off
							; 0 = not used
							; 0 = internal clock
							; 0 = timer off

	movwf	T1CON

	bsf 	INTCON, GIEL	; enable low priority interrupts
	bsf		INTCON, GIEH	; enambe high priority interrupts



;**************** STANDARD CODE MENU SELECTION *******************
			;Introduction

#ifdef  version1 
	movlw	.80			; display startup announcement line 1
	movwf	ptr_pos
	call	stan_char_1

	movlw	.96			; display startup announcement line 2
	movwf	ptr_pos
	call	stan_char_2

#endif

;	turn on timer 1 and timer 2 for self calibration
	bsf		T2CON , TMR2ON	; turn on the PWM timer2
	bsf		T1CON , TMR1ON	; turn on timer1


	bsf		GPS_verbose		;	turn off the extra GPS messages
	bsf		RCSTA , CREN	; turn on the GPS receiver
	bsf 	T0CON , TMR0ON	; turn on timer0

#ifdef skipgpscheck
#if ( skipgpscheck == 1 )
	bsf	test_mode
#endif
#endif	

	bsf	startM

menu



menu_tst_mode

#if ( cpuUsage==1)

	bsf		debugC	

#else

#if ( tst_mode_menu==1 )

#if	( statusLED==1 )

	bcf		debugC

#endif

	rcall	pause

#ifdef version1 
	movlw	.16				; test mode
	rcall	display_menu
#endif

tst_wait

stop_here
;	goto	stop_here

;	btfss	select
;	bra		tgl_tst_mde
;	btfsc	scroll
	bra		tst_wait
	btfss	scroll
	bra	$-2
	bra		end_tst_mode

tgl_tst_mde
	btg		test_mode
	btfss	select
	bra	$-2
	bra		tst_wait

#endif

#endif

end_tst_mode


;------------------ display GPS and set origin------------------
menu_gps

#if ( cpuUsage==0)

#if ( gps_menu==1)

#if	( statusLED==1 )
	bcf	debugC
#endif

	rcall	pause

#ifdef  version1 
	movlw	.48				;gps long lat
	rcall	display_menu
#endif

gps_wait
	btfss	select
	goto	display_GPS
	btfsc	scroll
	bra		gps_wait
	btfss	scroll
	bra	$-2

#endif

#endif

end_dsp_GPS



;------------------ display GPS course and speed------------------
menu_vtg

#if ( cpuUsage==0)

#if ( vtg_menu==1)

#if	( statusLED==1 )
	bcf	debugC
#endif

	rcall	pause

#ifdef version1
	movlw	.32				;gps vtg
	rcall	display_menu
#endif

vtg_wait
	btfss	select
	goto	display_vtg
	btfsc	scroll
	bra		vtg_wait
	btfss	scroll
	bra	$-2

#endif

#endif

end_dsp_vtg


;------------------ display offset ----------------------------------
menu_xy

#if ( cpuUsage==0)

#if ( xy_menu==1 )

#if	( statusLED==1 )
	bsf		debugC
#endif

	btfss	select			;wait for RB0 release
	bra	$-2		

#ifdef  version1 
	movlw	.112				;gps relative
	rcall	display_menu
#endif

xy_wait

	btfss	select
	goto	display_XY
	btfsc	scroll
	bra		xy_wait
	btfss	scroll
	bra	$-2

#endif

#endif

end_dsp_XY
;-------------------------------------------------------------------
	bra	menu			;begining of menu


pause
#ifdef	scroll
	btfss	scroll			;wait for RA4 release
	bra	$-2	
#endif
#ifdef	select	
	btfss	select			;wait for RB0 release
	bra	$-2	
#endif

	return

display_menu

#ifdef  version1 
	movwf	ptr_pos
	call	stan_char_1

	movlw	.64				;RA4=Next  RB0=Now
	movwf	ptr_pos
	goto	stan_char_2
#else
	return
#endif

flash_led

	btg	debugA
	btg	debugB
#ifdef	debugC
	btg	debugC
#endif

	return
	


;*******************************************************************

	GLOBAL	stan_table, start, end_dsp_GPS , end_dsp_XY , end_dsp_vtg

	end	
