

#include "processor.inc"
#include "autop_defines.inc"


	EXTERN	MULS2X1 , MULU2X1 , MULS2X2
;
;	get_bearing reads the compass, Steer decides where to point the front wheels.
;

PROG1 	CODE

rudd_cntrl
	bcf		RUD_req

;	retrieve filtered unsigned yaw rate gyro
	movff	yawU , ARG1H
	movff	yawH , ARG1L

;	subtract the baseline yaw
	movf	yaw0H , w
	subwf	ARG1L , f
	movf	yaw0U , w
	subwfb	ARG1H , f

;	add the baseline reference
	movf	ref0H , w
	addwf	ARG1L , f
	movf	ref0U , w
	addwfc	ARG1H , f

;	subtract reference
	movf	refH , w
	subwf	ARG1L , f
	movf	refU , w
	subwfb	ARG1H , f

;	multiply by yaw gyro gain
	movff	yawgain , ARG2L
	call	MULS2X1

;#ifdef	sf2520v1
;	doubleres
;	doubleres
;#endif

;	move to rudder accumulator
	movff	RES0 , ruddL
	movff	RES1 , ruddH
	movff	RES2 , ruddU
;
;	steering :
;
;	process only if modes request it
	btfss	GPS_steering
	bra		gps_is_off

	movf	actualDir, W			;	actual direction in W
	subwf	desiredDir, W			;	subtracts actual (W) from desired (F), answer in W
	movwf	errorDir				;	save the signed difference
	smult	strngGain , errorDir	;	signed multiply, error times steering gain
									;   16 times the answer ends up in PRODH:PRODL
;	add into the rudder accumulator
	movf	PRODL , w
	addwf	ruddL , f
	movf	PRODH , w
	addwfc	ruddH , f

;	extend the sign of the 2 byte result into 3 bytes
	movlw	0x0
	btfsc	PRODH , 7
	movlw	0xFF
	addwfc	ruddU , f

	bra		gps_is_on	

gps_is_off

;	add augmentation gain times pwrudder filtered minus the trim

	clrf	WREG
	movwf	ARG1H
	movff	pwrudfilH , ARG1L
	
	movf	strngTrim , w
	subwf	ARG1L , f
	clrf	WREG
	subwfb	ARG1H , f

	movff	auggain , ARG2L
	call	MULS2X1

	movf	RES0 , w
	addwf	ruddL , f
	movf	RES1 , w
	addwfc	ruddH , f
	movf	RES2 , w
	addwfc	ruddU , f	

gps_is_on

;	convert from an angle into a time period
	movlw	0x08
	addwf	ruddH , F
	movlw	0x00
	addwfc	ruddU , F

;	check upper byte for overflow
	movf	ruddU , w
	andlw	0xFF
	bz		ruddU_normal
	bn		rudd_min
	movlw	0xFF
	bra		rudd_trm_adjst

ruddU_normal

;	check high byte for overflow
	movf	ruddH, W				;	retrieve MSbyte of result
	andlw	0xF0					;	check for overflow
	bz		rudd_normal			;	no "clamping" needed
	bn		rudd_min				;	clamp to minimum for a "negative" time width pulse
	movlw	0xFF
	bra		rudd_trm_adjst

rudd_min
	clrf	WREG
	bra		rudd_trm_adjst

rudd_normal						;	unclamped case, must shift by 4 bits by using "swap:
	movlw	0xF0					;	mask for upper 4 bits
	andwf	ruddL, W				;	get the upper 4 bits of ruddL, put in W
	iorwf	ruddH, W				;	"or" the lower 4 bits of ruddH into W
									;	At this point the result is in W, except the nibbles are swapped.
	swapf	WREG					;	Swap the nibbles for the 4 bit shift.
;
;	here is where the trim comes in.
;	add the trim, test for overflow.
;
rudd_trm_adjst
;	convert rudder from period to angle
	btg		WREG , 7
	movff	pwrudin , strngTrimTemp
;	convert trim from period to angle
	btg		strngTrimTemp , 7
;	add the trim
	addwf	strngTrimTemp , W
	bov		ruddtrim_ov
;
;	convert from angle to time period
	btg		WREG , 7
	addwf	waggle , w				;	waggle the rudder during startup
	movwf	PWM1_dc					;	Move the intermediate result to its final destination.
	return

ruddtrim_ov
	btfss	WREG , 7
	bra		rudd_clamp_min
	bra		rudd_clamp_max
rudd_clamp_max
	movlw	0xFF					;	otherwise, clamp to maximum, if overflow and not negative
	movwf	PWM1_dc					;	result is the PWM duty cycle
	return
rudd_clamp_min
	movlw	0x00					;	clamp to minimum, result is PWM duty cycle
	movwf	PWM1_dc

	return

	GLOBAL rudd_cntrl

	end
