



#include "processor.inc"
#include "autop_defines.inc"

;********************************************************************************
;																				*
;	Interrupt serving and startup vector.										*
;																				*
;	serv_inter_H: 	service high priority interrupts.							*
;	serv_inter_L: 	service low priority interrupts.							*
;	start:		reboot starting vector.											*
;																				*
;	low priority interrupts include GPS USART and PWM timer 2.					*
;	low priority handling includes "manual" context save and restore of several	*
;	registers, as well as dispatching of the actual service routines.			*
;																				*
;	high priority interrupts will be used for time-critical events, including:	*
;	1.	"pass through" PWM control from the radio to a servo.					*
;	2.	measurement of pulse width for commands from radio.						*
;																				*
;********************************************************************************

	EXTERN	serv_GPS, serv_PWM, start, complete_GPS, state_machine

#if ( plane == 1)
	EXTERN 	elev_cntrl, rudd_cntrl
#endif

;	short block of code to vector startup and interrupt handling:

STARTUP CODE
	NOP
	goto	start			;	reboot
	ORG	0x08
	goto	serv_inter_H	;	high priority interrupt
	ORG 0x18
	goto	serv_inter_L	;	low priority interrupt
	NOP

PROG1 	CODE

TMR0Init
	movlw	B'00000110'		;
							;	0=timer off
							;	0=16 bit timer
							;	0=internal clock
							;	0=low to high
							;	0=use prescaler
							;	110=prescale, 128
	movwf	T0CON
	clrf	avail_flags
	clrf	cntrl_flags
	bsf		startM


	return

	GLOBAL	TMR0Init

serv_PORTB
;	movff	TMR2, TMR2_snap		; snapshot timer 2
	bcf		INTCON, RBIF		; clear the interrupt
	btfss	INTCON, RBIE
	return

	movff	PORTB , PORTB_snap	; snapshot the B port

	btfss	pwmin1
	bcf		pwmout1

	btfsc	pwmin1
	bsf		pwmout1

	btfss	pwmin2
	bcf		pwmout2

	btfsc	pwmin2
	bsf		pwmout2

	movf	PORTB_snap, W
	xorwf	PORTB_old , W
	movwf	PORTB_xor
	movff	PORTB_snap, PORTB_old

	btfsc	pwmin1_xor
	rcall	pwmin1_time
	btfsc	pwmin2_xor
	rcall	pwmin2_time

	return

pwmin1_time
	btfss	pwmin1
	bra		pwmin1_fall

	movff	tmr1Lsnap , tmrudinL
	movff	tmr1Hsnap , tmrudinH

;	movff	TMR2_snap , TRM1_t
	return
	
pwmin2_time
	btfss	pwmin2
	bra		pwmin2_fall

	movff	tmr1Lsnap , tmeleinL
	movff	tmr1Hsnap , tmeleinH

;	movff	TMR2_snap , TRM2_t
	return

pwmin1_fall

;	movff	TMR2_snap , TRM1_dc
;	movf	TRM1_t, W
;	subwf	TRM1_dc , F
;	movff	TMR2_snap , TRM1_t

	movf	tmrudinL , W		;	subtract rising edge time from the falling edge
	subwf	tmr1Lsnap , W
	movwf	tmrudinL

	movf	tmrudinH , W
	subwfb	tmr1Hsnap , W
	movwf	tmrudinH

	movf	tmrudinH , W	;	get the high order nibble
	andlw	0xF0
	movwf	tmrnible
	swapf	tmrnible

	movlw	0x01				;	 check for a pulse from 1 msed to 2 msec
	subwf	tmrnible , W
	bnz		rudinsat

	incf	pulsesrudin

	movlw	0x0F				; divide the 2 byte value by 16
	andwf	tmrudinH , F
	swapf	WREG , W
	andwf	tmrudinL , W
	iorwf	tmrudinH , W
	swapf	WREG , W
	movwf	pwrudin

	return

rudinsat
	movlw	0x00
	subwf	tmrnible , W
	bz		rudinmin
	movlw	0x02
	subwf	tmrnible , W
	bz		rudinmax
	return

rudinmin
	clrf	pwrudin
	incf	pulsesrudin
	return

rudinmax
	movlw	0xFF
	movwf	pwrudin
	incf	pulsesrudin	
	return

pwmin2_fall


	movf	tmeleinL , W		;	subtract rising edge time from the falling edge
	subwf	tmr1Lsnap , W
	movwf	tmeleinL

	movf	tmeleinH , W
	subwfb	tmr1Hsnap , W
	movwf	tmeleinH

	movf	tmeleinH , W	;	get the high order nibble
	andlw	0xF0
	movwf	tmrnible
	swapf	tmrnible

	movlw	0x01				;	 check for a pulse from 1 msed to 2 msec
	subwf	tmrnible , W
	bnz		eleinsat

	incf	pulseselein

	movlw	0x0F				; divide the 2 byte value by 16
	andwf	tmeleinH , F
	swapf	WREG , W
	andwf	tmeleinL , W
	iorwf	tmeleinH , W
	swapf	WREG , W
	movwf	pwelein

	return

eleinsat
	movlw	0x00
	subwf	tmrnible , W
	bz		eleinmin
	movlw	0x02
	subwf	tmrnible , W
	bz		eleinmax
	return

eleinmin
	clrf	pwelein
	incf	pulseselein
	return

eleinmax
	movlw	0xFF
	movwf	pwelein
	incf	pulseselein	
	return


serv_inter_H				;	high priority interrupt to measure PWM coming in

#if ( cpuUsage==1)
	bcf		debugC
#endif	

	movff	TMR1L , tmr1Lsnap	;	measure the time
	movff	TMR1H , tmr1Hsnap

	btfsc	INTCON, RBIF
	rcall	serv_PORTB

	btfsc	INTRISE
	rcall	serv_rise

	btfsc	INTFALL
	rcall	serv_fall

	btfsc	INTCON, TMR0IF
	bra		serv_TMR0

	retfie	FAST			;		reenable (high priority) interrupts and restore context

serv_rise

	bcf		INTRISE

	movff	tmr1Lsnap , tmselinL
	movff	tmr1Hsnap , tmselinH


	return

serv_fall

	bcf		INTFALL

	movf	tmselinL , W		;	subtract rising edge time from the falling edge
	subwf	tmr1Lsnap , W
	movwf	tmselinL

	movf	tmselinH , W
	subwfb	tmr1Hsnap , W
	movwf	tmselinH

	movf	tmselinH , W	;	get the high order nibble
	andlw	0xF0
	movwf	tmrnible
	swapf	tmrnible

	movlw	0x01				;	 check for a pulse from 1 msed to 2 msec
	subwf	tmrnible , W
	bnz		selinsat

	incf	pulsesselin

	movlw	0x0F				; divide the 2 byte value by 16
	andwf	tmselinH , F
	swapf	WREG , W
	andwf	tmselinL , W
	iorwf	tmselinH , W
	swapf	WREG , W
	movwf	pwselin

	return

selinsat
	movlw	0x00
	subwf	tmrnible , W
	bz		selinmin
	movlw	0x02
	subwf	tmrnible , W
	bz		selinmax
	return

selinmin
	clrf	pwselin
	incf	pulsesselin
	return

selinmax
	movlw	0xFF
	movwf	pwselin
	incf	pulsesselin	
	return

serv_TMR0
	bcf		INTCON, TMR0IF


;********************************************************************************
;																				*
;			TURN THE INTERRUPTS BACK ON!										*
;																				*
	bsf 	INTCON, GIEH		;	re-enable interrupts
;																				*
;																				*
;********************************************************************************

	btfss	GPS_tracking
	bra		not_armed

	bcf		GPS_tracking

armed

	bsf		nav_capable	
	goto	state_machine

not_armed

	bcf		GPS_tracking
	
	btfsc	test_mode
	bra		armed

	bcf		nav_capable
	goto	state_machine

serv_inter_L				;		low priority interrupt to handle PWM out and USART

#if (cpuUsage==1)
	bcf	debugC
#endif
								
								; 	save the background context onto the stack:
	movff	STATUS, PREINC2		; 	carry, zero, overflow, etc.
	movwf	PREINC2				;	WREG	
								;	service the PWM interrupt:
	btfsc	PIR1, TMR2IF		;	was there a timer 2 interrupt?
	call	serv_PWM
								;	service the GPS interrupt:
	btfsc	PIR1, RCIF			;	was there a USART interrupt?
	call	serv_GPS



;********************************************************************************
;																				*
;			TURN THE INTERRUPTS BACK ON!										*
;																				*
	bsf 	INTCON, GIEL		;	re-enable low priority interrupts
;																				*
;																				*
;********************************************************************************


	btfsc	GPS_req
	rcall	call_cmplt_GPS

#if ( plane == 1 )
	btfsc	ELE_req
	rcall	call_elev_cntrl
	btfsc	RUD_req
	rcall	call_rudd_cntrl
#endif

	movf	POSTDEC2, W
	movff	POSTDEC2, STATUS

	return

#if ( plane == 1)

saveMult
	movff	PRODL , PREINC2
	movff	PRODH , PREINC2
	movff	ARG1L , PREINC2
	movff	ARG1H , PREINC2
	movff	ARG2L , PREINC2
	movff	RES3  , PREINC2
	movff	RES2 , PREINC2
	movff	RES1 , PREINC2
	movff	RES0 , PREINC2
	return

restoreMult

	movff	POSTDEC2, RES0							
	movff	POSTDEC2, RES1							
	movff	POSTDEC2, RES2
	movff	POSTDEC2, RES3							
	movff	POSTDEC2, ARG2L							
	movff	POSTDEC2, ARG1H							
	movff	POSTDEC2, ARG1L							
	movff	POSTDEC2, PRODH							
	movff	POSTDEC2, PRODL
	return



call_elev_cntrl

	rcall	saveMult	
	call	elev_cntrl
	rcall	restoreMult
	return	

call_rudd_cntrl

	rcall	saveMult
	call	rudd_cntrl
	rcall	restoreMult
	return	

#endif						
	

call_cmplt_GPS

								;	save variables
	movff	PRODL, PREINC2		;	output of the multiplier
	movff	PRODH, PREINC2
	movf	FSR1L, W			;	low part of FSR1 indirect pointer
	movwf	PREINC2
	movf	FSR0L, W			;	low part of FSR0 indirect pointer
	movwf	PREINC2
	movff	TBLPTRL, PREINC2
	movff	TBLPTRH, PREINC2

	call	complete_GPS		;	complete the GPS processing

	movff	POSTDEC2, TBLPTRH
	movff	POSTDEC2, TBLPTRL							
	movf	POSTDEC2, W		
	movwf	FSR0L
	movf	POSTDEC2, W
	movwf	FSR1L
	movff	POSTDEC2, PRODH
	movff	POSTDEC2, PRODL
	return

	end	
