



#include "processor.inc"
#include "autop_defines.inc"

	EXTERN	set_gxx



PROG1 	CODE



radio_is_off


	bcf		radio_on
	bsf		radio_led
	movff	strngTrim , pwrudin
	movff	elevTrim , pwelein
	movlw	0xFF
	movwf	pwselin
	clrf	pulsesselin
	bcf		INTCON, RBIE	; turn off the pass through servo interrupt
	bra		det_state

state_machine

#if (	SELFTEST == 1 )
	echo_bit	SWR1 , st3st1
	echo_bit	SWR2 , st2
#endif

#if ( GPSDEBUG == 1)
	btg		PORTC,6	;
#else
	btfsc	GPS_verbose		;	GPS_verbose indicates the GPS sent a message we don't understand.
	call	set_gxx			;	set the options for the two GPS messages (GGA and VTG) that we want
#endif
;
;	first thing to do is to figure out if the radio is on or off.
;

	movf	pulsesselin , W
	andlw	0xF0
	bz		radio_is_off
	clrf	pulsesselin

		
radio_is_on


	bsf		radio_on			;	the radio is on
	bcf		radio_led
	bsf		INTCON, RBIE	; turn on the pass through servo interrupt

	movf	pwselin , w
	addlw	-(auto_pulse)

	bc		above_auto

	bsf		man_req
	bcf		auto_req
	bcf		circle_req

	bra		det_state

above_auto

	movf	pwselin , w
	addlw	-(circle_pulse)

	bc		above_circle

	bsf		auto_req
	bcf		man_req
	bcf		circle_req

	bra		det_state

above_circle

	bsf		circle_req
	bcf		auto_req
	bcf		man_req

det_state
;
;

	btfsc	startM
	bra		startS

	btfsc	calibrateM
	bra		calibrateS

	btfsc	acquiringM
	bra		acquiringS
	
	btfsc	manualM
	bra		manualS

	btfsc	autoM
	bra		autoS
	
	btfsc	returnM
	bra		returnS

	btfsc	circlingM
	bra		circlingS

	return	FAST

startS

	bra		ent_calibrateS
	return	FAST

calibrateS
;
;	activity:	decrement the timer
;
;	transitions:	on timeout, enter acquiring state
;
;
	btfss	radio_on
	bra		ent_calibrateS

#ifdef	gps_led
	btg		gps_led
#endif
	btg		mode_led

	decfsz	calib_timer
	return	FAST
	bra		ent_acquiringS

acquiringS
;
;	activity:	if nav capable, toggle gps led, decrement the timer
;
;
;	transitions:	on timeout, record the come-home origin and enter manualS
;
;

	btfss	nav_capable
	bra		ent_acquiringS

	btfss	radio_on
	bra		ent_calibrateS
#ifdef	gps_led
	btg		gps_led
#endif
	comf	waggle , f		;	waggle the rudder to confirm GPS

	decfsz	standby_timer
	return	FAST

;	[ record the come-home origin ]

	bsf		GPS_rq_org
	bra		ent_manualS

manualS
;
;	activity:	gps led depends on nav capable
#ifdef	gps_led
	echo_bit	nav_capable , gps_led
#endif

;
;	transitions:

;					radio off -> returnS
	btfss	radio_on
	bra		ent_returnS

;					circle_req & nav_capable -> snap circle origin and enter circling

	btfss	circle_req
	bra		m_check_auto
	btfss	nav_capable
	bra		m_check_auto

;	[ record the circling origin ]

	bsf		enable_focus
	bra		ent_circlingS

m_check_auto
;					auto_req ->	 autoS
	btfss	auto_req
	return	FAST
	bra		ent_autoS
;
;

	return	FAST

autoS
;
;	activity:	gps led depends on nav capable
;

#ifdef	gps_led
	echo_bit	nav_capable , gps_led
#endif

;	transitions:
;					radio off -> returnS

	btfss	radio_on
	bra		ent_returnS

;					circle_req & nav_capable -> snap circle origin and enter circling

	btfss	circle_req
	bra		a_check_man
	btfss	nav_capable
	bra		a_check_man

;	[ record the circling origin ]

	bsf		enable_focus
	bra		ent_circlingS

a_check_man

;					man_req -> manualS

	btfss	man_req
	return	FAST
	bra		ent_manualS
;
;

	return	FAST


returnS
;
;
;	activity:	gps led depends on nav capable


#ifdef	gps_led
	echo_bit	nav_capable , gps_led
#endif

;
;	transitions:
;					radio on -> manual state
	btfsc	radio_on
	bra		ent_manualS
;
	return	FAST

circlingS
;
;
;	activity:	gps led depends on nav capable


#ifdef	gps_led
	echo_bit	nav_capable , gps_led
#endif

;				toggle mode led
	btg		mode_led
;
;	transitions:
;					radio off -> returnS

	btfss	radio_on
	bra		ent_returnS

;					man_req -> manualS

	btfsc	man_req
	bra		ent_manualS

;					auto_req -> autoS

	btfsc	auto_req
	bra		ent_autoS
;
;

	return	FAST		
;
;	
;
ent_startS
	clrf	cntrl_flags
	clrf	waggle
	bsf		startM

	return	FAST

ent_calibrateS
	clrf	cntrl_flags
	clrf	waggle
	bsf		calibrateM

;	entry: 	intialize the calibrate timer

	movlw	calib_pause
	movwf	calib_timer

;			turn on the mode led

	bcf		mode_led

;			turn on the gps led
#ifdef	gps_led
	bcf		gps_led
#endif
;			put the servo control in pass-through

	clrf	CCP1CON			; turn off PWM control
	clrf	CCP2CON
	bcf		pwmout1			; turn off the outputs
	bcf		pwmout2

;			turn off gps steering

	bcf		GPS_steering
;


	return	FAST

ent_acquiringS
	clrf	cntrl_flags
	movlw	waggle0
	movwf	waggle
	bsf		acquiringM

;	entry:	initialize the aquire timer

	movlw	standby_pause
	movwf	standby_timer

;	set the leds
	bsf		mode_led
#ifdef	gps_led
	bcf		gps_led
#endif

;			set the trim

	movf	pwrudin	, W		; retrieve the pulse width from the radio for the rudder
	movwf	PWM1_dc			; initialize the duty cycle of the PWM control
	movwf	strngTrim		; set the steering trim

	movf	pwelein	, W		; retrive the pulse width from the radio for the elevator
	movwf	PWM2_dc			; initialize the duty cycle of the PWM control
	movwf	elevTrim		; set the speed trim

;			set the calibration

	bsf		calibpitch
	bsf		calibref	
	bsf		calibyaw	
	bsf		calibxaccel		
	bsf		calibyaccel
;			
	bcf		pwmout1			; turn off the outputs
	bcf		pwmout2

	movlw	0x0F
	movwf	CCP1CON			; turn on the PWM1 control
	movwf	CCP2CON			; turn on the PWM2 control

	return	FAST

ent_manualS
	clrf	cntrl_flags
	clrf	waggle
	bsf		manualM
;
;	entry:	place servos in pass through

	clrf	CCP1CON			; turn off PWM control
	clrf	CCP2CON
	bcf		pwmout1			; turn off the outputs
	bcf		pwmout2

;			mode led off

	bsf		mode_led

;			clear gps steering

	bcf		GPS_steering
;

	return	FAST

ent_autoS
	clrf	cntrl_flags
	clrf	waggle
	bsf		autoM
;
;	entry:	enable pwm control

	bcf		pwmout1			; turn off the outputs
	bcf		pwmout2

	movlw	0x0F
	movwf	CCP1CON			; turn on the PWM1 control
	movwf	CCP2CON			; turn on the PWM2 control

;			clear gps steering

	bcf		GPS_steering

;			mode led on

	bcf		mode_led
;

	return	FAST

ent_returnS
	clrf	cntrl_flags
	clrf	waggle
	bsf		returnM
;
;	entry:	enable pwm control

	bcf		pwmout1			; turn off the outputs
	bcf		pwmout2

	movlw	0x0F
	movwf	CCP1CON			; turn on the PWM1 control
	movwf	CCP2CON			; turn on the PWM2 control

;			set gps steering

	bsf		GPS_steering

;			mode led on

	bcf		mode_led

;			

	return	FAST

ent_circlingS
	clrf	cntrl_flags
	clrf	waggle
	bsf		circlingM
;
;	entry:	enable pwm control

	bcf		pwmout1			; turn off the outputs
	bcf		pwmout2

	movlw	0x0F
	movwf	CCP1CON			; turn on the PWM1 control
	movwf	CCP2CON			; turn on the PWM2 control

;			set gps steering

	bsf		GPS_steering

;			mode led on

	bcf		mode_led

;			

	return	FAST		
;


	GLOBAL	state_machine

	end
