



#include "processor.inc"
#include "autop_defines.inc"

	EXTERN	set_gxx


PROG1 	CODE

;	state diagram for the plane.
;	three states, manual, automatic, "quiet".
;	see the actual diagram so see how it works.
;	entry point is state machine.
;
;	state_machine is called on a regular (around once every 2 seconds) basis.
;	basically, these are "watchdog" services.

radio_is_off

;	it has been determined that the radio is off. clear the flag, and
;	determine what state is the "old" state.

	bcf		radio_on
	bra		det_state

state_machine

	btfsc	GPS_verbose			;	GPS_verbose indicates the GPS sent a message we don't understand.
	call	set_gxx				;	set the options for the two GPS messages (GGA and VTG) that we want
;
;	first thing to do is to figure out if the radio is on or off.
;
	movf	pulsePeriodH, W		;	check the pulse period against the minimum and maximum
	sublw	pp_min
	btfsc	STATUS, C			;	must be greater than pp_min
	bra		radio_is_off

	movf	pulsePeriodH, W
	sublw	pp_max
	btfss	STATUS, C			;	must be less than or equal to pp_max
	bra		radio_is_off

	movf	PWBuffH, W			;	check the pulse width against the minimum and maximum
	sublw	pw_min
	btfsc	STATUS, C			;	must be greater than pw_min
	bra		radio_is_off

	movf	PWBuffH, W
	sublw	pw_max
	btfss	STATUS, C			;	must be less than or equal to pw_max
	bra		radio_is_off		

	bsf		radio_on			;	the radio is on

det_state
;
;	determine what state the control is in.
;

	clrf	PWBuffH			;	radio measurements must be cleared,
	clrf	pulsePeriodH	;	because they "freeze" when the radio shuts off
;
;	check the control bits designating each possible state, and branch accordingly
;
	btfsc	quiet_mode
	bra		shut_down_s

	btfsc	auto_mode
	bra		auto_s

	btfsc	manual_mode
	bra		manual_s

	return	FAST		;	bullet proof, just in case no state gets set.
;
;	note: return FAST is used because this is a return from a HIGH priority interrupt.
;

shut_down_s
;
;	old state is the quiet state.
;
;	move to the manual state if the radio is on, and the man/auto switch is not set for auto.
;
	btfss	radio_on	;	is the radio on?
	return	FAST

	btfss	auto_req	;	is the man/switch not auto?
	bra		enter_manual

	return	FAST

auto_s
;
;	old state is the automatic state.
;
;	move to shut down if radio is off or if GPS (nav_capable) is lost.
;
;	move to manual if man/auto switch is not set to automatic.
;

#if	( statusLED==1 )
	btg		debugB	;	blink the auto LED in the auto mode
#endif

	btfss	radio_on
	bra		enter_shut_down	;	shut down if the radio is not end.

	btfss	nav_capable
	bra		enter_shut_down	;	shut down if the GPS is not working

	btfss	auto_req
	bra		enter_manual	;	enter manual if man/auto is not automatic

	return	FAST

manual_s
;
;	old state is the manual state.
;
;	if radio is lost, move to the "quiet" state.
;	if man/auto is not auto, leave in the manual state.
;
;	otherwise, if GPS is off, move to the "quiet" state.
;	otherwise, go to automatic mode.
;

	btfss	radio_on
	bra		enter_shut_down	;	shut down if radio is off

	btfss	auto_req
	return	FAST			;	do nothing if man/auto is not auto

	btfss	nav_capable
	bra		enter_shut_down	;	shut down if GPS is lost

	btfsc	nav_capable		;	this test is redundant, but should be left in case things are changed.
	bra		enter_auto		;	enter the automatic mode.

	return	FAST

enter_manual
;
;	new state is the manual state.
;
#if	( statusLED==1 )
	bsf		debugB	;	status LED is turned on for manual
#endif
;
;	set the manual mode bit and reset the other two
;
	bcf		quiet_mode
	bsf		manual_mode
	bcf		auto_mode
;
;	in the manual mode, the PWM control needs to be turned off.
;	this is down by clearing CCPXCON and pwmoutX, X=1,2
;
;	pass through control is turned on by simply enabling the pass through interrupt.
;
	clrf	CCP1CON			; turn off PWM control
	clrf	CCP2CON
	bcf		pwmout1			; turn off the outputs
	bcf		pwmout2
	bsf		INTCON, RBIE	; turn on the pass through servo interrupt

	return	FAST

enter_auto
;
;	new state is the automatic state.
;
#if	( statusLED==1 )
	bcf		debugB	;	shut off the man/auto LED
#endif
;
;	set the auto mode bit, reset the other 2.
;
	bcf		quiet_mode
	bcf		manual_mode
	bsf		auto_mode
;
	clrf	desiredSpeed	; make sure the power to the drive is off to start with
;	
	movf	TRM1_dc , W		; retrieve the current manual trim setting, channel 1
	movwf	PWM1_dc			; initialize the duty cycle of the PWM control
	btg		WREG , 7
	movwf	strngTrim		; set the steering trim
;
	movf	TRM2_dc , W		; retrieve the current manual trim setting, channel 1
	movwf	PWM2_dc			; initialize the duty cycle of the PWM control
	btg		WREG , 7
	movwf	speedTrim		; set the speed trim
;
	bcf		INTCON, RBIE	; turn off the pass through servo interrupt
	bcf		pwmout1			; turn off the outputs
	bcf		pwmout2

	movlw	0x0F
	movwf	CCP1CON			; turn on the PWM1 control
	movwf	CCP2CON			; turn on the PWM2 control

	return	FAST

enter_shut_down
;
;	new state is the "quiet" mode.
;
#if	( statusLED==1 )
	bcf		debugB	;	clear the LED indication
#endif
;
;	set the "quiet" mode bit, reset the other 2.
;
	bsf		quiet_mode
	bcf		manual_mode
	bcf		auto_mode
;
;	for the plane, pass through control is enabled in the "shut down" state
;	pass through control is turned on by simply enabling the pass through interrupt.
;
	clrf	CCP1CON			; turn off PWM control
	clrf	CCP2CON
	bcf		pwmout1			; turn off the outputs
	bcf		pwmout2
	bsf		INTCON, RBIE	; turn on the pass through servo interrupt

	return	FAST

	GLOBAL	state_machine

	end
